#Copyright (c) 2015 3D Control Systems LTD

#3DPrinterOS client is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#3DPrinterOS client is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with 3DPrinterOS client.  If not, see <http://www.gnu.org/licenses/>.

# Author: Vladimir Avdeev <another.vic@yandex.ru>, Oleg Panasevych <panasevychol@gmail.com>

import logging
import time

class CheckerWaiter:

    CHECK_PERIOD = 0.1
    NAME_FOR_LOGGING = 'place your name for logging here'

    def __init__(self, app, check_function):
        self.check_function = check_function
        self.logger = logging.getLogger(self.__class__.__name__)
        self.logger.setLevel('INFO')
        self.app = app
        self.ignore_flag = False
        self.waiting = not self.check()

    def check(self):
        return bool(self.check_function())

    def ignore(self):
        self.logger.info('Ignoring action: ' + self.NAME_FOR_LOGGING)
        self.ignore_flag = True
        time.sleep(self.CHECK_PERIOD) # this action is necessary to continue right after wait loop is over

    def wait(self):
        self.logger.info('Waiting for an action: ' + self.NAME_FOR_LOGGING)
        while not self.app.stop_flag and not self.ignore_flag:
            if self.check():
                break
            time.sleep(self.CHECK_PERIOD)
        self.waiting = False
        self.logger.info('...end of waiting: ' + self.NAME_FOR_LOGGING)