#!/usr/bin/env python

#Copyright (c) 2015 3D Control Systems LTD

#3DPrinterOS client is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#3DPrinterOS client is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with 3DPrinterOS client.  If not, see <http://www.gnu.org/licenses/>.

# Author: Vladimir Avdeev <another.vic@yandex.ru>

import os
import re
import sys
import tempfile
import subprocess
import zipfile
import platforms

import paths


windows_process_name = 'pythonw.exe'
unix_process_name_re = re.compile(".*python.*(launcher\.py|app\.py)")
pid_file_name = "3dprinteros.pid"
tmp_path = tempfile.gettempdir()
pid_file_path = os.path.join(tmp_path, pid_file_name)
update_file_path = paths.UPDATE_FILE_PATH
update_script_file_name = 'update_script.py'


def install_update():
    print 'Updating 3DPrinterOS'
    try:
        z = zipfile.ZipFile(update_file_path)
        z.extractall()
        z.close()
    except zipfile.BadZipfile:
        os.remove(update_file_path)
        print 'Update failed!\nReason: bad zip file'
        return True
    except Exception as e:
        print 'Update failed!\nReason: error while extracting.\nDetails: ' + str(e)
    else:
        os.remove(update_file_path)
        print 'Update successful!'
        if os.path.exists(update_script_file_name):
            subprocess.call([sys.executable, update_script_file_name])
            os.remove(update_script_file_name)
        open(os.path.join(paths.CURRENT_SETTINGS_FOLDER, 'update_complete'), "w")
        return True

def run_command(command):
    process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
    output = process.stdout.readlines()
    process.stdout.close()
    return output

def get_nix_processes():
    return run_command('ps ax')

def get_win_processes():
    processes_list = run_command("tasklist /svc")
    return [proc for proc in processes_list if len(proc.split()) > 1]

def check_process_name_by_pid(pid):
    print "Checking process name"
    if sys.platform.startswith('win'):
        for task in get_win_processes():
            words = task.split()
            if len(words) > 1:
                current_task_pid = words[1]
                if current_task_pid == pid:
                    return words[0] == windows_process_name
    elif sys.platform.startswith('linux') or sys.platform.startswith('darwin'):
        for task in get_nix_processes():
            words = task.split()
            if words[0] == pid and len(words) > 5:
                process = words[4] + words[5]
                return unix_process_name_re.match(process)
    else:
        raise RuntimeError("Your OS is not supported by 3DPrinterOS")

def get_process_list():
    print "Getting process list"
    if sys.platform.startswith('win'):
        processes = get_win_processes()
        pids = map(lambda x: x.split()[1], processes)
    elif sys.platform.startswith('linux') or sys.platform.startswith('darwin'):
        processes = get_nix_processes()
        pids = map(lambda x: x.split()[0], processes)
    else: raise RuntimeError("Your OS is not supported by 3DPrinterOS")
    return pids

def execute_custom_args():
    if len(sys.argv) > 1:
        args = sys.argv[1:]
        if '--restore_default_settings' in args:
            import config
            os.remove(config.Config.USER_SETTINGS_FILE)
            print "Default settings were restored"
        if '--logout' in args:
            import user_login
            user_login.logout()
            print "Logout successful"
        if '--clear_logs' in args:
            import log
            log.clear_logs()
            print "Log files were erased"
        if '--erase_all_data':
            for root, dirs, files in os.walk(paths.CURRENT_SETTINGS_FOLDER, topdown=False):
                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    os.rmdir(os.path.join(root, name))
            os.rmdir(paths.CURRENT_SETTINGS_FOLDER)
            print "3DPrinterOS's data folder was erased"
        sys.exit(0)

def restart():
    print "Restarting 3DPrinterOS"
    python = sys.executable
    args = [python, python, __file__]
    if platforms.get_platform() == "win":
        args[1] = '"' + args[1] + '"'
        args[2] = '"' + args[2] + '"'
    os.execl(*args)

def run():
    execute_custom_args()
    print "Launching 3DPrinterOS"
    with open(pid_file_path, "w") as f:
        f.write(str(os.getpid()))
    if os.path.exists(update_file_path):
        if not install_update():
            "Update failed. Can't start."
            return
    try:
       import app
    except ImportError as e:
        print "Can't import some modules: " + e.message
    else:
        printeros_app = app.App()
        need_restart = printeros_app.start_main_loop()
        os.remove(pid_file_path)
        if need_restart:
            restart()

def open_browser_tab():
    import app
    app.App.open_browser()

if __name__ == "__main__":
    try:
        f = open(pid_file_path)
    except IOError:
        run()
    else:
        pid_in_file = f.read()
        f.close()
        if pid_in_file in get_process_list() and check_process_name_by_pid(pid_in_file):
            print "3DPrinterOS is already running - opening browser window"
            open_browser_tab()
        else:
            run()




