import io
import sys
import time
import signal
import logging

import log
import config
import http_client
from dual_cam import DualCameraMaster


class RPiCamera(DualCameraMaster):

    def __init__(self):
        self.logger = log.create_logger('camera')
        logging.basicConfig()
        self.stop_flag = False
        signal.signal(signal.SIGINT, self.intercept_signal)
        signal.signal(signal.SIGTERM, self.intercept_signal)
        self.http_client = http_client.HTTPClient(self, keep_connection_flag=True)
        self.init_user_token()
        self.min_loop_time = config.get_settings()["camera"]["min_loop_time"]
        try:
            import picamera
        except ImportError:
            logging.info("Can't import picamera module. Please install it with apt-get install python-picamera")
        try:
            self.capture = picamera.PiCamera()
            self.logger.info("Got connection to PiCamera")
        except Exception:
            self.logger.warning("Can't init PiCamera. Camera seems to be not connected or not enabled in RPi config.")
        else:
            self.main_loop()

    def init_user_token(self):
        if len(sys.argv) > 2:
            self.user_token = sys.argv[1]
            mac = sys.argv[2]
        else:
            import user_login
            mac = None
            login = user_login.UserLogin(self)
            login.wait()
            self.user_token = login.user_token
        if mac:
            self.http_client.mac = mac  # we need to use MAC from client to ensure that it's not changed on camera restart

    def main_loop(self):
        time.sleep(self.min_loop_time)
        while not self.stop_flag:
            frame_start_time = time.time()
            stream = io.BytesIO()
            self.capture.capture(stream, format='jpeg')
            frame = stream.getvalue()
            self.send_frame(1, frame)
            while time.time() < frame_start_time + 0.001 + self.min_loop_time:
                time.sleep(0.01)
        self.capture.close()
        self.http_client.close()
        sys.exit(0)

if __name__ == "__main__":
    RPiCamera()
